// Copyright 2024 SquareBlock Inc. All Rights Reserved.
// Author: tianlei.richard@qq.com (tianlei.richard)

#pragma once

#include <array>
#include <memory>

#include "common.h"
#include "phone_material.h"
#include "triangle.h"

class Mesh {
public:
  Mesh(const std::vector<std::shared_ptr<Vertex>> &vertices,
       const std::vector<Triangle> &primitives,
       const std::shared_ptr<PhoneMaterial> &phone_material);

  static std::vector<Mesh> load_mesh(const std::string &file_path);

public:
  Point2d get_texture_coordinate(const unsigned int index) const;
  Point3d get_normal_vector(const unsigned int index) const;

  const std::vector<Triangle> &get_primitives() const { return primitives_; };
  std::vector<Triangle> &get_primitives() { return primitives_; };

  std::shared_ptr<PhoneMaterial> &get_material() { return material_; }
  const std::shared_ptr<PhoneMaterial> &get_material() const {
    return material_;
  }

private:
  std::vector<std::shared_ptr<Vertex>> vertices_;
  std::vector<Triangle> primitives_;

  std::shared_ptr<PhoneMaterial> material_;
};
